local MothSpawner = Class(function(self, inst)
    self.inst = inst
    self.inst:StartUpdatingComponent(self)
    self.moths = {}
    self.timetospawn = 20
    self.nexttimetospawn = 20
    self.mothcap = 3
    self.nummoths = 0
    self.followplayer = true
    self.prefab = "moth"
end)

function MothSpawner:SetMoth(moth)
    self.prefab = moth
end

function MothSpawner:GetSpawnPoint(spawnerinst,spawntype)
	local rad = 25
	local x,y,z = spawnerinst.Transform:GetWorldPosition()
	local nearby_ents = TheSim:FindEntities(x,y,z, rad, {spawntype})
	local mindistance = 36
	local validferns = {}
	for k,fern in ipairs(nearby_ents) do
		if fern and
		spawnerinst:GetDistanceSqToInst(fern) > mindistance then
			table.insert(validferns, fern)			
		end
	end

	if #validferns > 0 then
		local f = validferns[math.random(1, #validferns)]
		return f
	else
		return nil
	end
end

function MothSpawner:StartTracking(inst)
    inst.persists = false
    if not inst.components.homeseeker then
	    inst:AddComponent("homeseeker")
	end

	self.moths[inst] = function()
	    if self.moths[inst] then
	        inst:Remove()
	    end
	end

	self.inst:ListenForEvent("entitysleep", self.moths[inst], inst)
	
	self.nummoths = self.nummoths + 1
end

function MothSpawner:StopTracking(inst)
    inst.persists = true
	inst:RemoveComponent("homeseeker")
	if self.moths[inst] then
		self.inst:RemoveEventCallback("entitysleep", self.moths[inst], inst)
		self.moths[inst] = nil
		self.nummoths = self.nummoths - 1
	end
end

function MothSpawner:OnUpdate( dt )
	local spawnerinst
    local day = GetClock():IsDay()

    if self.followplayer then
    	spawnerinst = GetPlayer()
    else
    	spawnerinst = self.inst
    end

    if spawnerinst then
	    
		if self.timetospawn > 0 then
			self.timetospawn = self.timetospawn - dt
		end
	    
	    local sm = GetSeasonManager()
		local forestspawn = not day and not sm:IsWinter()
		if SaveGameIndex:IsModeShipwrecked() then
			local forestspawn = not day and not sm:IsWetSeason()			
		end

		if spawnerinst and (GetWorld():IsCave() or forestspawn) and self.prefab then
			if self.timetospawn <= 0 then
				local spawnFern = nil
				if math.random() >= 0.25 then
					spawnFern = self:GetSpawnPoint(spawnerinst,"flower")
				else
					spawnFern = self:GetSpawnPoint(spawnerinst,"cave_fern")
				end
				if spawnFern and self.nummoths < self.mothcap then
					local moth = SpawnPrefab(self.prefab)
					local spawn_point = Vector3(spawnFern.Transform:GetWorldPosition() )
					moth.Physics:Teleport(spawn_point.x,spawn_point.y,spawn_point.z)
					moth.components.pollinator:Pollinate(spawnFern)
					self:StartTracking(moth)
					moth.components.homeseeker:SetHome(spawnFern)
				end
				if self.followplayer then
					self.timetospawn = self.nexttimetospawn + math.random()*self.nexttimetospawn
				else
					self.timetospawn = math.random()
				end
			end
		end
	end
    
end

function MothSpawner:GetDebugString()
	return "Next spawn: "..tostring(self.timetospawn)
end

function MothSpawner:OnSave()
	return 
	{
		timetospawn = self.timetospawn,
		nexttimetospawn = self.nexttimetospawn,
    	mothcap = self.mothcap,
	}
end

function MothSpawner:OnLoad(data)
	self.timetospawn = data.timetospawn or 10
	self.nexttimetospawn = data.nexttimetospawn or 10
	self.mothcap = data.mothcap or 4
	if self.mothcap <= 0 then
		self.inst:StopUpdatingComponent(self)
	end
end

function MothSpawner:SpawnModeNever()
	self.timetospawn = -1
	self.nexttimetospawn = 10
    self.mothcap = 0
    self.inst:StopUpdatingComponent(self)
end

function MothSpawner:SpawnModeVeryHeavy()
	self.timetospawn = 0.3
	self.nexttimetospawn = 0.3
    self.mothcap = 15
end

function MothSpawner:SpawnModeHeavy()
	self.timetospawn = 20
	self.nexttimetospawn = 6
    self.mothcap = 7
end

function MothSpawner:SpawnModeMed()
	self.timetospawn = 20
	self.nexttimetospawn = 12
    self.mothcap = 4
end

function MothSpawner:SpawnModeLight()
	self.timetospawn = 20
	self.nexttimetospawn = 40
    self.mothcap = 1
end

return MothSpawner
