require "prefabutil"

local assets=
{
	Asset("ANIM", "anim/bird_cage.zip"),

	Asset("ANIM", "anim/crow_build.zip"),
	Asset("ANIM", "anim/robin_build.zip"),
	Asset("ANIM", "anim/robin_winter_build.zip"),
	Asset("ANIM", "anim/owl_build.zip"),
	Asset("ANIM", "anim/owl_winter_build.zip"),
	Asset("ANIM", "anim/beardbird_build.zip"),
	Asset("ANIM", "anim/shadow_trans.zip"),
}

local prefabs =
{
	"bird_egg",

	-- everything it can "produce" and might need symbol swaps from
	"crow",
	"robin",
	"robin_winter",
	"collapse_small",
	"owl",
}

local bird_symbols=
{
	"crow_beak",
	"crow_body",
	"crow_eye",
	"crow_leg",
	"crow_wing",
	"tail_feather",
}

local function BecomeOwl(inst)
	if not inst.isowl or inst.iswinterowl then
		for k,v in pairs(bird_symbols) do
			local build = "owl_build"
			inst.AnimState:OverrideSymbol(v, build, v)
		end
		inst.isowl = true
		inst.iswinterowl = false
		inst.beardling = false
		inst.components.sanityaura.aura = 0
		local light = inst.entity:AddLight()
		light:Enable(false)
		--inst.sounds = owlsounds
	end
end

local function BecomeBeardling(inst)
	if not inst.beardling then
		for k,v in pairs(bird_symbols) do
			local build = "beardbird_build"
			inst.AnimState:OverrideSymbol(v, build, v)
		end
		inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED		
		inst.isowl = false
		inst.iswinterowl = false
		inst.beardling = true
		local light = inst.entity:AddLight()
		light:Enable(false)
		--inst.sounds = beardsounds
	end
end

local function BecomeShadow(inst)
	if inst.isowl or inst.iswinterowl or inst.beardling then
		for k,v in pairs(bird_symbols) do
			local build = "shadow_trans"
			inst.AnimState:OverrideSymbol(v, build, v)
		end
		inst.isowl = false
		inst.iswinterowl = false
		inst.beardling = false
		inst.components.sanityaura.aura = -TUNING.SANITYAURA_LARGE
			local light = inst.entity:AddLight()
			light:Enable(true)
			light:SetFalloff(0.5)
			light:SetIntensity(.7)
			light:SetRadius(1.25)
			light:SetColour(255/255, 81/255, 87/255)
		--inst.sounds = beardsounds
	end
end

local function DonWinterFur(inst)
	if not inst.iswinterowl or inst.isowl then
		for k,v in pairs(bird_symbols) do
			local build = "owl_winter_build"
			inst.AnimState:OverrideSymbol(v, build, v)
		end
		inst.isowl = false
		inst.iswinterowl = true
		inst.beardling = false
		inst.components.sanityaura.aura = 0
		local light = inst.entity:AddLight()
		light:Enable(false)
		--inst.sounds = owlsounds
	end
end

local function CheckTransformState(inst)
	local player = GetPlayer()
	if player.components.sanity:GetPercent() > TUNING.BEARDLING_SANITY then
		if not GetSeasonManager() or not GetSeasonManager():IsWinter() then
			BecomeOwl(inst)
		else
			DonWinterFur(inst)
		end
	elseif player.components.sanity:GetPercent() <= 0.15 then
		BecomeShadow(inst)	
	else
		BecomeBeardling(inst)		
	end
end

local function ShouldAcceptItem(inst, item)
	local seed_name = string.lower(item.prefab .. "_seeds")
	local can_accept = item.components.edible and (Prefabs[seed_name] or item.prefab == "seeds" or item.components.edible.foodtype == "MEAT")
	if inst.birdtype == "owl" then
		if item.prefab == "rabbit" or item.prefab == "mole" then can_accept = item.prefab == "rabbit" or item.prefab == "mole" 
		else can_accept = item.components.edible and item.components.edible.foodtype == "MEAT" end
	end
	
	if item.prefab == "egg" or item.prefab == "bird_egg" or item.prefab == "furry_egg" or item.prefab == "shadow_egg" or item.prefab == "rottenegg" or item.prefab == "monstermeat" then
		can_accept = false
	end
	
	if  inst.birdtype == "owl" and item.prefab == "monstermeat" and not (inst.isowl or inst.iswinterowl) then
		can_accept = item.prefab == "monstermeat"
	end
	return can_accept
end

local function OnRefuseItem(inst, item)
	inst.AnimState:PlayAnimation("flap")
    inst.SoundEmitter:PlaySound("dontstarve/birds/wingflap_cage")
    inst.AnimState:PushAnimation("idle_bird")
	if inst.birdtype == "owl" then
		CheckTransformState(inst)
	end
end

local function OnGetItemFromPlayer(inst, giver, item)

	if inst.components.sleeper and inst.components.sleeper:IsAsleep() then
		inst.components.sleeper:WakeUp()
	end

    if item.components.edible or item.prefab == "rabbit" or item.prefab == "mole" then
		local seed_name = string.lower(item.prefab .. "_seeds")
		
		local can_accept = nil
		if inst.birdtype == "owl" then
			CheckTransformState(inst)
			if item.prefab == "rabbit" or item.prefab == "mole" then can_accept = item.prefab == "rabbit" or item.prefab == "mole" 
			else can_accept = item.components.edible and item.components.edible.foodtype == "MEAT" end
		else
			can_accept = Prefabs[seed_name] or item.prefab == "seeds" or item.components.edible.foodtype == "MEAT"
		end
		if can_accept then

			local seedspawnprefab = "seeds"

			inst.AnimState:PlayAnimation("peck")
			inst.AnimState:PushAnimation("peck")
			inst.AnimState:PushAnimation("peck")
			inst.AnimState:PushAnimation("hop")
			
			if item.prefab == "rabbit" then
					inst.SoundEmitter:PlaySound("dontstarve/rabbit/scream_short")
			elseif item.prefab == "mole" then
					inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/mole/death")
			end
			
			inst.AnimState:PushAnimation("idle_bird", true)
			inst:DoTaskInTime(60*FRAMES, function() 
				if item.prefab == "rabbit" or item.prefab == "mole" then
					if inst.beardling then
						inst.components.lootdropper:SpawnLootPrefab("furry_egg")
						if math.random() <= 0.25 then
							inst.components.lootdropper:SpawnLootPrefab("beardhair")	
						end
					elseif inst.isowl or inst.iswinterowl then
						inst.components.lootdropper:SpawnLootPrefab("bird_egg")
						if math.random() <= 0.25 then
							inst.components.lootdropper:SpawnLootPrefab("boneshard")	
						end
					else
						inst.components.lootdropper:SpawnLootPrefab("shadow_egg")
						if math.random() <= 0.25 then
							inst.components.lootdropper:SpawnLootPrefab("nightmarefuel")	
						end
					end
		    	elseif item.components.edible.foodtype == "MEAT" then
					
					if inst.birdtype == "owl" then
						if inst.beardling then
							inst.components.lootdropper:SpawnLootPrefab("furry_egg")
						elseif inst.isowl or inst.iswinterowl then
							inst.components.lootdropper:SpawnLootPrefab("bird_egg")
						else
							inst.components.lootdropper:SpawnLootPrefab("shadow_egg")
						end
					else
						inst.components.lootdropper:SpawnLootPrefab("bird_egg")
					end
				else
				    if Prefabs[seed_name] then
				    	local num_seeds = math.random(2)
				    	for k = 1, num_seeds do
						    local loot = inst.components.lootdropper:SpawnLootPrefab(seed_name)
				    	end
				    	
						if math.random() < .5 then
							inst.components.lootdropper:SpawnLootPrefab(seedspawnprefab)
						end
				    else
						inst.components.lootdropper:SpawnLootPrefab(seedspawnprefab)
				    end
				end

			    if inst.components.occupiable and inst.components.occupiable.occupant 
			    and inst.components.occupiable.occupant:IsValid() and inst.components.occupiable.occupant.components.perishable then
			    	print("EAT!")
		            inst.components.occupiable.occupant.components.perishable:SetPercent(1)		         
		        end
		    end)
			
		end
    end
end

local function DoIdle(inst)
	if inst.birdtype == "owl" then
		CheckTransformState(inst)
	end
	local r = math.random()
	if r < .5 then
        inst.AnimState:PlayAnimation("caw")
        if inst.chirpsound then
			inst.SoundEmitter:PlaySound(inst.chirpsound)
		end
	elseif r < .6 then
        inst.AnimState:PlayAnimation("flap")
        inst.SoundEmitter:PlaySound("dontstarve/birds/wingflap_cage")
	else
        inst.AnimState:PlayAnimation("hop")
	end
    inst.AnimState:PushAnimation("idle_bird")
end

local function StopIdling(inst)
	if inst.birdtype == "owl" then
		CheckTransformState(inst)
	end
    if inst.idletask then
        inst.idletask:Cancel()
        inst.idletask = nil
    end
end

local function StartIdling(inst)
	if inst.birdtype == "owl" then
		CheckTransformState(inst)
	end
    inst.idletask = inst:DoPeriodicTask(6, DoIdle)
end

local function ShouldSleep(inst)
	if inst.components.occupiable:IsOccupied() then
	   return GetClock():IsNight()
	else
		return false
	end
end

local function ShouldWake(inst)
    return GetClock():IsDay()
end

local function NocturnalSleep(inst)
	if inst.components.occupiable:IsOccupied() then
		CheckTransformState(inst)
	   return GetClock():IsDay()
	else
		return false
	end
end

local function NocturnalWake(inst)
	local nightcheck = not GetClock():IsDay()
	CheckTransformState(inst)
    return nightcheck
end


local function onoccupied(inst, bird)

	if bird.components.perishable then
		bird.components.perishable:StopPerishing()
	end
	
    inst:AddComponent("sleeper")
	inst:AddComponent("eater")  

	inst.components.trader:Enable()
	
	if bird.prefab == "owl" then
		inst.components.sleeper:SetSleepTest(NocturnalSleep)
		inst.components.sleeper:SetWakeTest(NocturnalWake)
		inst.birdtype = "owl"
		CheckTransformState(inst)
		inst.CheckTransformState = CheckTransformState
		BecomeOwl(inst)
	else
		for k,v in pairs(bird_symbols) do
			local build = bird.prefab .. "_build"
			if bird.trappedbuild then
				build = bird.trappedbuild
			end
			inst.AnimState:OverrideSymbol(v, build, v)
		end
	    inst.components.sleeper:SetSleepTest(ShouldSleep)
		inst.components.sleeper:SetWakeTest(ShouldWake)
		inst.components.eater:SetBird()
		inst.birdtype = bird.prefab
	end 

	if bird.prefab == "parrot_pirate" then
		inst.AnimState:Show("HAT")
	else
		inst.AnimState:Hide("HAT")
	end	
	
	inst.chirpsound = bird.sounds and bird.sounds.chirp
	inst.AnimState:PlayAnimation("flap")
	inst.AnimState:PushAnimation("idle_bird", true)
	StartIdling(inst)
end

local function onemptied(inst, bird)
	inst:RemoveComponent("sleeper")
	
	StopIdling(inst)
	inst.components.trader:Disable()
	for k,v in pairs(bird_symbols) do
		inst.AnimState:ClearOverrideSymbol(v)
	end
	local light = inst.entity:AddLight()
	light:Enable(false)
	inst.components.sanityaura.aura = 0
	inst.isowl = false
	inst.iswinterowl = false
	inst.beardling = false
	inst.AnimState:Hide("HAT")
	inst.AnimState:PlayAnimation("idle", true)
end

local function onhammered(inst, worker)
	--[[
	inst.components.container:DropEverything()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	--]]
	if inst.components.occupiable:IsOccupied() then
		local item = inst.components.occupiable:Harvest()
		if item then
			item.Transform:SetPosition(inst.Transform:GetWorldPosition())
			item.components.inventoryitem:OnDropped()
		end
	end
	inst.components.lootdropper:DropLoot()
	inst:Remove()
	
end

local function onhit(inst, worker)

	if inst.components.occupiable:IsOccupied() then
		inst.AnimState:PlayAnimation("hit_bird")
		inst.AnimState:PushAnimation("flap")
        inst.SoundEmitter:PlaySound("dontstarve/birds/wingflap_cage")
		inst.AnimState:PushAnimation("idle_bird", true)
	else
		inst.AnimState:PlayAnimation("hit_idle")
		inst.AnimState:PushAnimation("idle")
	end
	--inst.components.container:Close()
end


local function testfn(inst, guy)
	return guy:HasTag("bird")
end
		
local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle")
	inst.SoundEmitter:PlaySound("dontstarve/common/craftable/birdcage_craft")
end

local function GoToSleep(inst)
	if inst.components.occupiable:IsOccupied() then
		StopIdling(inst)
		inst.AnimState:PlayAnimation("sleep_pre")
		inst.AnimState:PushAnimation("sleep_loop", true)
	end
end

local function WakeUp(inst)
	if inst.components.occupiable:IsOccupied() then
		inst.AnimState:PlayAnimation("sleep_pst")
		inst.AnimState:PushAnimation("idle_bird", true)
		StartIdling(inst)
	end
end


local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    MakeObstaclePhysics(inst, .5 )
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "birdcage.png" )
    
	if inst.birdtype == "owl" then
		CheckTransformState(inst)
		inst.CheckTransformState = CheckTransformState
		BecomeOwl(inst)
	end
	
    inst.AnimState:SetBank("birdcage")
    inst.AnimState:SetBuild("bird_cage")
    inst.AnimState:PlayAnimation("idle")

    inst.AnimState:Hide("HAT")

    inst:AddTag("structure")
    inst:AddComponent("inspectable")
    
    inst:AddComponent("lootdropper")
	
	inst:AddComponent("sanityaura")

	inst.AnimState:Hide("HAT")
    
    inst:AddComponent("occupiable")
    inst.components.occupiable.occupytestfn = testfn
    inst.components.occupiable.onoccupied = onoccupied
    inst.components.occupiable.onemptied = onemptied

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)    
	
	inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
	inst.components.trader.onaccept = OnGetItemFromPlayer
	inst.components.trader.onrefuse = OnRefuseItem
	inst.components.trader:Disable()
	MakeSnowCovered(inst, .01)	
	inst:ListenForEvent( "onbuilt", onbuilt)  
		
		inst.OnSave = function(inst, data)
			data.isowl = inst.isowl or nil
			data.iswinterowl = inst.iswinterowl or nil
			data.birdtype = inst.birdtype or nil
		end        
		
		inst.OnLoad = function(inst, data)
			if data and data.birdtype == "owl" then
				if not data.isowl and not data.iswinterowl then
					BecomeBeardling(inst)
				elseif not data.isowl and data.iswinterowl then
					DonWinterFur(inst)
				end
			end
		end
	
    inst:ListenForEvent("gotosleep", function(inst) GoToSleep(inst) end)
    inst:ListenForEvent("onwakeup", function(inst) WakeUp(inst) end)
	
    return inst
end

return Prefab( "common/birdcage", fn, assets, prefabs),
		MakePlacer("common/birdcage_placer", "birdcage", "bird_cage", "idle") 

