local assets=
{
	Asset("ANIM", "anim/furry_eggs.zip"),
	Asset("ANIM", "anim/shadow_eggs.zip"),
}


local prefabs =
{
    "rottenegg",
}    

local function commonfn()
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
    inst.AnimState:SetBank("birdegg")
	inst.AnimState:SetBuild("furry_eggs")
	inst.AnimState:PlayAnimation("idle")
    
    MakeInventoryPhysics(inst)
    
    inst:AddComponent("edible")
    inst.components.edible.foodtype = "MEAT"

	inst:AddTag("monstermeat")
	
	if SaveGameIndex:IsModeShipwrecked() then
    MakeInventoryFloatable(inst, "idle_water", "idle")
	MakeBlowInHurricane(inst, TUNING.WINDBLOWN_SCALE_MIN.MEDIUM, TUNING.WINDBLOWN_SCALE_MAX.MEDIUM)
	end
    
	inst:AddComponent("perishable")
	inst.components.perishable:SetPerishTime(TUNING.PERISH_FAST)
	inst.components.perishable:StartPerishing()
	inst.components.perishable.onperishreplacement = "rottenegg"
    
    
    inst:AddComponent("stackable")
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("bait")
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
    
    inst:AddComponent("tradable")

    return inst
end

local function defaultfn()
	local inst = commonfn()
	
    inst.components.inventoryitem.imagename = "furry_egg"
    inst.components.inventoryitem.atlasname = "inventoryimages/furry_egg.xml"
    
    inst.components.edible.healthvalue = -TUNING.HEALING_MEDSMALL
    inst.components.edible.sanityvalue = -TUNING.SANITY_MED
    inst.components.edible.hungervalue = TUNING.CALORIES_SMALL
    inst.components.perishable:SetPerishTime(TUNING.PERISH_MED)
    
	
    inst:AddComponent("cookable")
    inst.components.cookable.product = "furry_egg_cooked"

	if SaveGameIndex:IsModeShipwrecked() then
    inst:AddComponent("appeasement")
    inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_SMALL
	end
	
	inst.components.tradable.goldvalue = 2
	
	return inst
end

local function shadowfn()
	local inst = commonfn()
    inst.AnimState:SetBuild("shadow_eggs")
	
    inst.components.inventoryitem.imagename = "shadow_egg"
    inst.components.inventoryitem.atlasname = "inventoryimages/shadow_egg.xml"

    inst.AnimState:PlayAnimation("idle")
    
    inst.components.edible.healthvalue = -TUNING.CALORIES_TINY
    inst.components.edible.sanityvalue = -TUNING.SANITY_HUGE
    inst.components.perishable:SetPerishTime(TUNING.PERISH_SUPERSLOW)

	if SaveGameIndex:IsModeShipwrecked() then
    inst:AddComponent("appeasement")
    inst.components.appeasement.appeasementvalue = TUNING.APPEASEMENT_MED
	end
	
	inst.components.tradable.goldvalue = 3

	return inst
end

local function cookedfn()
	local inst = commonfn()

    inst.AnimState:PlayAnimation("cooked")
	
    inst.components.inventoryitem.imagename = "furry_egg_cooked"
    inst.components.inventoryitem.atlasname = "inventoryimages/furry_egg_cooked.xml"

    inst.components.edible.foodstate = "COOKED"
    inst.components.edible.healthvalue = -TUNING.HEALING_SMALL
    inst.components.edible.sanityvalue = -TUNING.SANITY_TINY
    inst.components.edible.hungervalue = TUNING.CALORIES_MEDSMALL
    inst.components.perishable:SetPerishTime(TUNING.PERISH_FAST)
    inst.components.perishable.onperishreplacement = "spoiled_food"    

	return inst
end

return Prefab("common/inventory/furry_egg", defaultfn, assets, prefabs),
		Prefab("common/inventory/furry_egg_cooked", cookedfn, assets),
		Prefab("common/inventory/shadow_egg", shadowfn, assets, prefabs)

