--[[
birds.lua

Different birds are just reskins of crow without any special powers at the moment.
To make a new bird add it at the bottom of the file as a 'makebird(name)' call

This assumes the bird already has a build, inventory icon, sounds and a feather_name prefab exists

]]--

	    local assets=
    {
	    Asset("ANIM", "anim/crow.zip"),
	    Asset("ANIM", "anim/owl_build.zip"),
		Asset("ANIM", "anim/owl_winter_build.zip"),
		Asset("ANIM", "anim/beardbird_build.zip"),
		Asset("ANIM", "anim/shadow_build.zip"),
	    Asset("SOUND", "sound/owls.fsb"),
		Asset("ATLAS", "inventoryimages/owl.xml"),
		Asset("ATLAS", "inventoryimages/owl_winter.xml"),
		Asset("ATLAS", "inventoryimages/furry_bird.xml"),
		Asset("ATLAS", "inventoryimages/shadow_bird.xml"),
    }
	
    local owlsounds = 
    {
        takeoff = "owls/owls/takeoff_owl",
        chirp = "owls/owls/chirp_owl",
		flyin = "dontstarve/birds/flyin",
    }
	local shadowsounds = 
    {
        takeoff = "dontstarve/sanity/creature2/dissappear",
        chirp = "dontstarve/sanity/creature2/attack",
		flyin = "dontstarve/sanity/creature2/appear",
    }

local function BecomeOwl(inst)
	if not inst.isowl or inst.iswinterowl then
		inst.AnimState:SetBuild("owl_build")
		inst.components.lootdropper:SetLoot{}
        inst.components.lootdropper:AddRandomLoot("feather_crow", 1)
        inst.components.lootdropper:AddRandomLoot("smallmeat", 1)
        inst.components.lootdropper.numrandomloot = 1
		inst.isowl = true
		inst.iswinterowl = false
		inst.beardling = false
		inst.components.sanityaura.aura = 0
		inst.components.inventoryitem.atlasname = "inventoryimages/owl.xml"
		inst.components.inventoryitem:ChangeImageName("owl")
		inst.name = "Owl"
		inst:RemoveTag("shadow")
        inst:RemoveTag("shadowcreature")
		inst:RemoveTag("NOCLICK")
		inst.components.transparentonsanity:IsActive(false)	
		local light = inst.entity:AddLight()
		light:Enable(false)
		inst.sounds = owlsounds
	end
end

local function BecomeBeardling(inst)
	if not inst.beardling then
		inst.AnimState:SetBuild("beardbird_build")
		inst.components.lootdropper:SetLoot{}
		inst.components.lootdropper:AddRandomLoot("beardhair", 0.5)	    
		inst.components.lootdropper:AddRandomLoot("monstermeat", 1)	    
		inst.components.lootdropper:AddRandomLoot("nightmarefuel", 0.5)	  
		inst.components.lootdropper.numrandomloot = 1  		
		inst.isowl = false
		inst.iswinterowl = false
		inst.beardling = true
		inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
		inst.components.inventoryitem.atlasname = "inventoryimages/furry_bird.xml"
		inst.components.inventoryitem:ChangeImageName("furry_bird")
		inst.name = "Owl"
		inst:RemoveTag("shadow")
        inst:RemoveTag("shadowcreature")
		inst:RemoveTag("NOCLICK")
		inst.components.transparentonsanity:IsActive(false)
		local light = inst.entity:AddLight()
		light:Enable(false)
		inst.sounds = owlsounds
	end
end

local function BecomeShadow(inst)
	if inst.isowl or inst.iswinterowl or inst.beardling then
		inst.AnimState:SetBuild("shadow_build")
		inst.components.lootdropper:SetLoot{}	    	    
		inst.components.lootdropper:AddRandomLoot("nightmarefuel", 1)
		if math.random() >= 0.5 then
		inst.components.lootdropper.numrandomloot = 1   
		end		
		inst.isowl = false
		inst.iswinterowl = false
		inst.beardling = false
		inst.components.sanityaura.aura = -TUNING.SANITYAURA_LARGE
		inst.components.inventoryitem.atlasname = "inventoryimages/shadow_bird.xml"
		inst.components.inventoryitem:ChangeImageName("shadow_bird")
		inst.name = ""
		inst:AddTag("shadow")
        inst:AddTag("shadowcreature")
		inst:AddTag("NOCLICK")
		inst.components.transparentonsanity:IsActive(true)
		if not inst.components.inventoryitem.owner then
			local light = inst.entity:AddLight()
			light:Enable(true)
			light:SetFalloff(0.5)
			light:SetIntensity(.7)
			light:SetRadius(1.25)
			light:SetColour(255/255, 81/255, 87/255)
		end
		inst.sounds = shadowsounds
	end
end

local function DonWinterFur(inst)
	if not inst.iswinterowl or inst.isowl then
		inst.AnimState:SetBuild("owl_winter_build")
		inst.components.lootdropper:SetLoot{}
        inst.components.lootdropper:AddRandomLoot("feather_robin_winter", 1)
        inst.components.lootdropper:AddRandomLoot("smallmeat", 1)
        inst.components.lootdropper.numrandomloot = 1
		inst.isowl = false
		inst.iswinterowl = true
		inst.beardling = false
		inst.components.sanityaura.aura = 0
		inst.components.inventoryitem.atlasname = "inventoryimages/owl_winter.xml"
		inst.components.inventoryitem:ChangeImageName("owl_winter")
		inst.name = "Owl"
		inst:RemoveTag("shadow")
        inst:RemoveTag("shadowcreature")
		inst:RemoveTag("NOCLICK")
		inst.components.transparentonsanity:IsActive(false)
		local light = inst.entity:AddLight()
		light:Enable(false)
		inst.sounds = owlsounds
	end
end

local function CheckTransformState(inst)
	if not inst.components.health:IsDead() then
		local player = GetPlayer()
		if player.components.sanity:GetPercent() > TUNING.BEARDLING_SANITY then
			if not GetSeasonManager():IsWinter() then
				BecomeOwl(inst)
			else
				DonWinterFur(inst)
			end
		elseif player.components.sanity:GetPercent() <= 0.15 then
			BecomeShadow(inst)	
		else
			BecomeBeardling(inst)		
		end
	end
end

local function TrackInSpawner(inst)
    local ground = GetWorld()
    if ground and ground.components.owlspawner then
        ground.components.owlspawner:StartTracking(inst)
    end
end

local function StopTrackingInSpawner(inst)
    local ground = GetWorld()
    if ground and ground.components.owlspawner then
        ground.components.owlspawner:StopTracking(inst)
    end
end

local function ShouldSleep(inst)
	--CheckTransformState(inst)
    return GetClock():IsDay() and not inst.sg:HasStateTag("flying")
end

local function ondrop(inst)
	inst.sg:GoToState("stunned")
	CheckTransformState(inst)
	if not (inst.isowl or inst.iswinterowl or inst.beardling) then
		local light = inst.entity:AddLight()
		light:Enable(true)
		light:SetFalloff(0.5)
		light:SetIntensity(.7)
		light:SetRadius(1.5)
		light:SetColour(255/255, 81/255, 87/255)
	else
		local light = inst.entity:AddLight()
		light:Enable(false)
	end
end

local function OnWake(inst)
	CheckTransformState(inst)
	inst.checktask = inst:DoPeriodicTask(10, CheckTransformState)
end

local function OnSleep(inst)
	 if inst.checktask then
		inst.checktask:Cancel()
		inst.checktask = nil
	 end
end

local function GetCookProductFn(inst)
	if inst.isowl or inst.iswinterowl then
		return "cookedsmallmeat" 
	elseif inst.beardling then
		return "cookedmonstermeat"
	else
		return "nightmarefuel"
	end
end

local function OnAttacked(inst, data)
    local x,y,z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x,y,z, 30, {'bird'})
    
    local num_friends = 0
    local maxnum = 5
    for k,v in pairs(ents) do
        if v ~= inst then
            v:PushEvent("gohome")
            num_friends = num_friends + 1
        end
        
        if num_friends > maxnum then
            break
        end
        
    end
end

local function seedspawntest()
	return not GetWorld().components.seasonmanager:IsWinter()
end

local function makebird(name, soundname)
 
	
    local prefabs =
    {
        "seeds",
        "smallmeat",
        "cookedsmallmeat",
        --"feather_"..name,
    }

    local function OnTrapped(inst, data)
        if data and data.trapper and data.trapper.settrapsymbols then
			CheckTransformState(inst)
            data.trapper.settrapsymbols(name.."_build")
        end
    end


    local function canbeattacked(inst, attacked)
        return not inst.sg:HasStateTag("flying")
    end

    local function fn()
	    local inst = CreateEntity()
	    local trans = inst.entity:AddTransform()
	    local anim = inst.entity:AddAnimState()
	    local sound = inst.entity:AddSoundEmitter()
	    inst.sounds = owlsounds
        inst.entity:AddPhysics()
        inst.Transform:SetTwoFaced()
	    local shadow = inst.entity:AddDynamicShadow()
        shadow:SetSize( 1, .75 )
	    shadow:Enable(false)

        inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
        inst.Physics:ClearCollisionMask()
        inst.Physics:CollidesWith(COLLISION.WORLD)
        inst.Physics:SetSphere(1)
        inst.Physics:SetMass(1)
        
        inst:AddTag("bird")
        inst:AddTag("owl")
        inst:AddTag("smallcreature")
        
        anim:SetBank("crow")
        anim:SetBuild("owl_build")
        anim:PlayAnimation("idle")
        inst.trappedbuild = "owl_build"
        
        inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
        inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	    inst.components.locomotor:SetTriggersCreep(false)
        inst:SetStateGraph("SGbird")

        inst:AddComponent("lootdropper")
        inst.components.lootdropper:AddRandomLoot("feather_crow", 1)
        inst.components.lootdropper:AddRandomLoot("smallmeat", 1)
        inst.components.lootdropper.numrandomloot = 1
        
        inst:AddComponent("occupier")
        
        inst:AddComponent("eater")
        inst.components.eater:SetCarnivore()
        
        inst:AddComponent("sleeper")
        inst.components.sleeper:SetSleepTest(ShouldSleep)

        inst:AddComponent("inventoryitem")
        inst.components.inventoryitem.nobounce = true
        inst.components.inventoryitem.canbepickedup = false
        --inst.components.inventoryitem:SetOnDroppedFn(ondrop)
		inst.components.inventoryitem.atlasname = "inventoryimages/owl.xml"
		inst.components.inventoryitem.imagename = "owl"

        inst:AddComponent("cookable")
		inst.components.cookable.product = GetCookProductFn

		inst:AddComponent("transparentonsanity")
		inst.components.transparentonsanity:IsActive(false)

        inst:AddComponent("combat")
        inst.components.combat.hiteffectsymbol = "crow_body"
        inst.components.combat.canbeattackedfn = canbeattacked
        inst:AddComponent("health")
        inst.components.health:SetMaxHealth(TUNING.BIRD_HEALTH)
        inst.components.health.murdersound = "dontstarve/wilson/hit_animal"
        
        inst:AddComponent("inspectable")
       
        local brain = require "brains/owlbrain"
        inst:SetBrain(brain)
		
		inst:AddComponent("sanityaura")
		
        MakeSmallBurnableCharacter(inst, "crow_body")
        MakeTinyFreezableCharacter(inst, "crow_body")
		
        --inst:AddComponent("periodicspawner")
        --inst.components.periodicspawner:SetPrefab("seeds")
        --inst.components.periodicspawner:SetDensityInRange(40, 2)
        --inst.components.periodicspawner:SetMinimumSpacing(8)
		--inst.components.periodicspawner:SetSpawnTestFn( seedspawntest )

        inst.TrackInSpawner = TrackInSpawner
        
        inst:ListenForEvent("ontrapped", OnTrapped)
        inst:ListenForEvent("onremove", StopTrackingInSpawner)
        inst:ListenForEvent("enterlimbo", StopTrackingInSpawner)

		BecomeOwl(inst)
		CheckTransformState(inst)
		inst.CheckTransformState = CheckTransformState 
		
		inst.OnSave = function(inst, data)
			data.isowl = inst.isowl or nil
			data.iswinterowl = inst.iswinterowl or nil
			data.beardling = inst.beardling
		end        
		
		inst.OnEntityWake = OnWake
		inst.OnEntitySleep = OnSleep    
	
		
		inst.OnLoad = function(inst, data)
			if data then
				if not data.isowl and not data.iswinterowl then
					if data.beardling then
						BecomeBeardling(inst)
					else
						BecomeShadow(inst)
					end
				elseif not data.isowl and data.iswinterowl then
					DonWinterFur(inst)
				end
			end
		end
		
        inst:ListenForEvent("attacked", OnAttacked)

		MakeFeedablePet(inst, TUNING.TOTAL_DAY_TIME*2, nil, ondrop)
        return inst
    end
    
    return Prefab("forest/animals/owl", fn, assets, prefabs)
end

return makebird("owl", "owl")
       
